fibonacci :: Int -> Int
fibonacci 0 = 0
fibonacci 1 = 1
fibonacci n = fibonacci (n - 1) + fibonacci (n - 2)

main :: IO ()
main = mapM_ (\i -> putStrLn $ "fib(" ++ show i ++ ") = " ++ show (fibonacci i)) [0..10]
