"""Create a textual"""

from clippt.app import PresentationApp
from clippt.slides import load, md, py, sh, slide

__all__ = [
    "PresentationApp",
    "md",
    "sh",
    "py",
    "load",
    "slide",
]
